#include "Frontend Vars.h"
#include "Play.h"

#undef EXTERN
#define EXTERN extern

#include <AIFF.h>

//==============================================================  Functions

//--------------------------------------------------------------  SetVolume
//void PLAYER_StopSound( int theChannel, int theVolume)
//{
//	theVolume = theVolume/(0x8000/kFullVolume);
//}

//--------------------------------------------------------------  SetVolume
void PLAYER_SetVolume( int theChannel, int Volume)
{
SndCommand	theCommand;
OSErr		theErr;

	Volume >>= 7;
	theCommand.param1 = 0;
	theCommand.param2 = (Volume<<16) | Volume;
	theCommand.cmd = volumeCmd;						// Send quietCmd to stop any current sound.
	theErr = SndDoImmediate( externalChannel[theChannel], &theCommand);
}

//--------------------------------------------------------------  StopSound
void PLAYER_StopSound( int theChannel)
{
SndCommand	theCommand;
OSErr		theErr;

	theCommand.param1 = 0;
	theCommand.param2 = 0L;
	theCommand.cmd = quietCmd;						// Send quietCmd to stop any current sound.
	theErr = SndDoImmediate( externalChannel[theChannel], &theCommand);

	theCommand.cmd = flushCmd;						// Send flushCmd to clear the sound queue.
	theErr = SndDoImmediate( externalChannel[theChannel], &theCommand);

	externalPlaying[ theChannel] = false;
}

//--------------------------------------------------------------  PlaySound
void PLAYER_PlaySound( Handle theSound, int theChannel, Fixed fixed, Boolean bRepeatFlag)
{
SndCommand	theCommand;
OSErr		theErr;
ExtSoundHeader ExtSndHeaders;
	
	
	theCommand.cmd = quietCmd;						// Send quietCmd to stop any current sound.
	theCommand.param1 = 0;
	theCommand.param2 = 0L;
	theErr = SndDoImmediate( externalChannel[theChannel], &theCommand);
	
	theCommand.cmd = flushCmd;						// Send 1st a flushCmd to clear the sound queue.
	theErr = SndDoImmediate( externalChannel[theChannel], &theCommand);

	ExtSndHeaders.samplePtr = (Ptr)((char*)*theSound);	//point to sample buffer
	ExtSndHeaders.numChannels = 1;						//mono
	ExtSndHeaders.sampleRate = rate11025hz;
	ExtSndHeaders.loopStart = 0;						//start of sample repeat
	if( bRepeatFlag)
		ExtSndHeaders.loopEnd = GetHandleSize( theSound)-1;	//end of sample to repeat
	else
		ExtSndHeaders.loopEnd = 0;
	
	ExtSndHeaders.encode = extSH;						//extended sound header
	ExtSndHeaders.baseFrequency = 60;					//not used here
	ExtSndHeaders.numFrames = GetHandleSize( theSound);
	//ExtSndHeaders.AIFFSampleRate = 0;			
	//ldtox80(&AIFFRate, &(**hSounds[i]).AIFFSampleRate);	//unused here
	ExtSndHeaders.markerChunk = nil;					//unused here
	ExtSndHeaders.instrumentChunks = nil;				//unused here
	ExtSndHeaders.AESRecording = nil;					//unused here
	ExtSndHeaders.sampleSize = (short)0x0008;			//16 bits per sample

	theCommand.cmd = bufferCmd;							// Then, send a bufferCmd to channel 1.
	theCommand.param1 = 0;								// The sound played will be soundID.
	theCommand.param2 = (long)&ExtSndHeaders;
	theErr = SndDoImmediate( externalChannel[theChannel], &theCommand);
	if( theErr == noErr)
		{
		externalPlaying[ theChannel] = true;
//		if( !bRepeatFlag)
			{
			theCommand.cmd = callBackCmd;					// Lastly, queue up a callBackCmd to notify us
			theCommand.param1 = kSoundDone[theChannel];		// when the sound has finished playing.
			theCommand.param2 = SetCurrentA5();
			theErr = SndDoCommand( externalChannel[theChannel], &theCommand, TRUE);
			}
		}
}

//--------------------------------------------------------  PlayExternalSound
int PLAYER_PlayExternalSound (Handle theSound, Fixed rate, Boolean bRepeatFlag)
{
int		a;

	for( a=0; a<MAXIMUM_CHANNELS; a++)
		if( externalPlaying[a] == false)
			{
			PLAYER_PlaySound( theSound, a, rate, bRepeatFlag);
			return a;
			}
	return -1;
}

//--------------------------------------------------------  ExternalCallBack
// Callback routine.  If this looks ugly, blame Apple's Universal Headers.
// The callback routine is called after a sound finishes playing.  The
// callback routine is extremely useful in that it enables us to know when
// to set the sound channels priority back to 0 (meaning no sound playing).
// Keep in mind (by the way) that this funciton is called at interrupt time
// and thus may not cause memory to be moved.  Also, note that also because
// of the interupt situation, we need to handle setting A5 to point to our
// app's A5 and then set it back again.

RoutineDescriptor ExternalCallBackRD = BUILD_ROUTINE_DESCRIPTOR(uppSndCallBackProcInfo, PLAYER_ExternalCallBack);

pascal void PLAYER_ExternalCallBack (SndChannelPtr theChannel, SndCommand *theCommand)
{
int 	a;
#ifndef __POWERPC__
long	thisA5, gameA5;
	
	gameA5 = theCommand->param2;			// Extract our A5 from sound command.
	thisA5 = SetA5(gameA5);					// Point A5 to our app (save off current A5).
#endif

	for( a=0; a<MAXIMUM_CHANNELS; a++)
		if (theCommand->param1 == kSoundDone[a])	// See if it's either of OUR callback.
			externalPlaying[a] = 0;

#ifndef __POWERPC__
	thisA5 = SetA5(thisA5);	
#endif
}

//--------------------------------------------------------  InitSound
void PLAYER_InitSound (void)
{
OSErr theErr;
int   a;
	
	#if USESROUTINEDESCRIPTORS
		externalCallBackUPP = &ExternalCallBackRD;	// Handle Universal Header ugliness.
	#else
		externalCallBackUPP = (SndCallBackUPP) &ExternalCallBack;
	#endif
	
	theErr = noErr;									// Assume no errors.
	
	for( a=0; a<MAXIMUM_CHANNELS; a++)
		{
		externalPlaying[a] = false;
		kSoundDone[a] = SOUND_RANDOM+a;
		theErr = SndNewChannel(&externalChannel[a], sampledSynth, initNoInterp + initMono, (SndCallBackUPP)externalCallBackUPP);
		if (theErr != noErr)							// See if it worked.
			externalChannel[a] = NULL;
		}
}

//--------------------------------------------------------  KillSound
void PLAYER_KillSound (void)
{
OSErr	theErr, thisErr;
int		a;

	theErr = noErr;
	for( a=0; a<MAXIMUM_CHANNELS; a++)
		{
		PLAYER_StopSound( a);
		thisErr = SndDisposeChannel(externalChannel[a], TRUE);
	  	externalChannel[a] = 0;
	    if( thisErr != noErr)
			theErr = thisErr;
		}
}

