//	------------------------------------------------------------------------------------------	
//
//		Copyright 1995, 1996, Interplay Productions, Inc.
//		Not to be distributed without express consent of Interplay Productions
//		You can use this code in MacPlay games, but nowhere else.
//
//		Written by Chris De Salvo
//
//	------------------------------------------------------------------------------------------	

//	------------------------------	Includes

#include "CD Utils.h"

//	------------------------------	Private Definitions

#define CD_DRIVER_NAME			"\p.AppleCD"

//	CD addressing types
#define addressLBS				0x0000
#define addressMIN_SEC_FRAME	0x0001
#define addressTrack			0x0002

//	CD Driver csCodes

#define csDriveStatus			8
#define csReadTOC				100
#define csReadQ				101
#define csAudioTrackSearch		103
#define csPlayAudio				104
#define csAudioStop				106
#define csGetSpindleSpeed		113

//	------------------------------	Private Types
//	------------------------------	Private Variables

static short	gCDDriverRef = 0;
static short	gCurrentTrack = 0;

//	------------------------------	Private Functions

static long NumToBCD(short theNum);

//	------------------------------	Public Variables

//	--------------------	OpenCD

OSErr
CDOpen()
{
	return (OpenDriver(CD_DRIVER_NAME, &gCDDriverRef));
}

//	--------------------	GetCDSpeed

short
CDGetSpeed()
{
OSErr		theErr;
ParamBlockRec	thePB;

	thePB.cntrlParam.ioCompletion = nil;
	thePB.cntrlParam.ioCRefNum = gCDDriverRef;
	thePB.cntrlParam.csCode = csGetSpindleSpeed;
	
	theErr = PBControlSync(&thePB);

	if (theErr)
		return (-1);
	else
		return (thePB.cntrlParam.csParam[0]);
}

//	--------------------	PlayCDAudioTrack

OSErr
CDPlayAudioTrack(char trackNumber, char playMode)
{
ParamBlockRec	thePB;

	gCurrentTrack = trackNumber;

	thePB.cntrlParam.ioCompletion = nil;							//	Perform call synchronously
	thePB.cntrlParam.ioCRefNum = gCDDriverRef;					//	CD driver ref num
	
	thePB.cntrlParam.csCode = csPlayAudio;						//	csCode to seek and play audio selection
	
	thePB.cntrlParam.csParam[0] = addressTrack;					//	Use track number addressing
	thePB.cntrlParam.csParam[1] = 0;							//	High word of Track in BCD
	thePB.cntrlParam.csParam[2] = LoWord(NumToBCD(trackNumber));	//	Low word of Track in BCD
	thePB.cntrlParam.csParam[3] = 0;							//	This is a start position
	thePB.cntrlParam.csParam[4] = playMode;

	thePB.cntrlParam.csParam[5] = 0;							//	These fields must be zero
	thePB.cntrlParam.csParam[6] = 0;
	thePB.cntrlParam.csParam[7] = 0;
	thePB.cntrlParam.csParam[8] = 0;
	thePB.cntrlParam.csParam[9] = 0;
	thePB.cntrlParam.csParam[10] = 0;
	
	return (PBControlSync(&thePB));
}

//	--------------------	StopCDAudioPlayback

OSErr
CDStopAudioPlayback(void)
{
ParamBlockRec	thePB;
short		theTrack;
OSErr		theErr;

	theErr = CDGetCurrentAudioTrack(&theTrack);
	if (theErr)
		return (theErr);

	thePB.cntrlParam.ioCompletion = nil;							//	Perform call synchronously
	thePB.cntrlParam.ioCRefNum = gCDDriverRef;					//	CD driver ref num
	
	thePB.cntrlParam.csCode = csPlayAudio;
	
	thePB.cntrlParam.csParam[0] = addressTrack;					//	Use track number addressing
	thePB.cntrlParam.csParam[1] = 0;							//	High word of Track in BCD
	thePB.cntrlParam.csParam[2] = theTrack;					//	Low word of Track in BCD

	thePB.cntrlParam.csParam[3] = 0;							//	These fields must be zero
	thePB.cntrlParam.csParam[4] = 0;
	thePB.cntrlParam.csParam[5] = 0;
	thePB.cntrlParam.csParam[6] = 0;
	thePB.cntrlParam.csParam[7] = 0;
	thePB.cntrlParam.csParam[8] = 0;
	thePB.cntrlParam.csParam[9] = 0;
	thePB.cntrlParam.csParam[10] = 0;
	
	return (PBControlSync(&thePB));
}

//	--------------------	NumToBCD

static long
NumToBCD(short theNum)
{
unsigned long	theBCD;

	theBCD = 0x0000;
	
	//	Get the digits
	theBCD |= theNum % 10;
	theNum /= 10;

	//	Get the tens
	theBCD |= (theNum % 10) << 4;
	theNum /= 10;
	
	//	Get the hundreds
	theBCD |= (theNum % 10) << 8;
	theNum /= 10;	

	//	Get the thousands
	theBCD |= (theNum % 10) << 12;
	
	return (theBCD);
}

//	--------------------	BCDToNum

static short
BCDToNum(unsigned char bcd)
{
unsigned char	place;
short		num = 0;
	
	place = bcd & 0x0F;									//	Get the first 10 bits
	num += place;
	
	place = (bcd >> 4) & 0x0F;
	num += (place * 10);
	
	return (num);
}

//	--------------------	GetCurrentCDAudioTrack

OSErr
CDGetCurrentAudioTrack(short *theTrack)
{
OSErr		theErr;
ParamBlockRec	thePB;

	thePB.cntrlParam.ioCompletion = nil;						//	Perform call synchronously
	thePB.cntrlParam.ioCRefNum = gCDDriverRef;				//	CD driver ref num
	
	thePB.cntrlParam.csCode = csReadQ;						//	Get the position of the optical pickup
	
	thePB.cntrlParam.csParam[0] = 0;						//	These fields must be zero
	thePB.cntrlParam.csParam[1] = 0;
	thePB.cntrlParam.csParam[2] = 0;
	thePB.cntrlParam.csParam[3] = 0;
	thePB.cntrlParam.csParam[4] = 0;
	thePB.cntrlParam.csParam[5] = 0;
	thePB.cntrlParam.csParam[6] = 0;
	thePB.cntrlParam.csParam[7] = 0;
	thePB.cntrlParam.csParam[8] = 0;
	thePB.cntrlParam.csParam[9] = 0;
	thePB.cntrlParam.csParam[10] = 0;
	
	theErr = PBControlSync(&thePB);
	
	*theTrack =  BCDToNum(thePB.cntrlParam.csParam[0] & 0xFF);	//	Return the current track number

	return (theErr);
}

//	------------------------------	IsFileOnCD

IsOnCDResult
CDIsFileOnCD(short fileRef)
{
HParamBlockRec	volRec;
DCtlHandle		devHandle;
FCBPBRec			fileRec;
OSErr			theErr;

StringPtr			driverName;
StringPtr			cdDriverName = "\p.AppleCD";

	//	Get information on this open file
	fileRec.ioCompletion = nil;
	fileRec.ioFCBIndx = 0;
	fileRec.ioVRefNum = 0;
	fileRec.ioRefNum = fileRef;
	fileRec.ioNamePtr = nil;

	theErr = PBGetFCBInfoSync(&fileRec);
	if (theErr)
		return (isOnCDErr);

	//	Get information about the volume of this file
	volRec.volumeParam.ioCompletion = nil;
	volRec.volumeParam.ioNamePtr = nil;
	volRec.volumeParam.ioVRefNum = fileRec.ioFCBVRefNum;
	volRec.volumeParam.ioVolIndex = 0;
	
	theErr = PBHGetVInfoSync(&volRec);
	if (theErr)
		return (isOnCDErr);

	//	Get info on this drive's driver
	devHandle = GetDCtlEntry(volRec.volumeParam.ioVDRefNum);

	if (! devHandle)
		return (isOnCDErr);
		
	if (! ((**devHandle).dCtlFlags & dRAMBased))
		driverName = (StringPtr) ((*(DCtlPtr) devHandle).dCtlDriver + 18);
	else
		driverName = (StringPtr) ((**devHandle).dCtlDriver + 18);
		
	if (EqualString(driverName, cdDriverName, false, false) == true)
		return (isOnCDTrue);

	return (isOnCDFalse);
}

//	------------------------------	IsCDInserted

Boolean
CDIsCDInserted()
{
ParamBlockRec	params;
OSErr		theErr;
	
	//	On return from a drive status call the csParam fields have the same info
	//	as the regular disk driver DrvSts structure.  The fourth byte in that
	//	structure is true or false depending on whether or not a disk is in the drive.

	params.cntrlParam.ioCompletion = nil;
	params.cntrlParam.ioVRefNum = 1;
	params.cntrlParam.ioCRefNum = gCDDriverRef;
	params.cntrlParam.csCode = csDriveStatus;
	
	theErr = PBStatusSync((ParmBlkPtr) &params);
	if (theErr)
		return (false);
	
	return (params.cntrlParam.csParam[1] & 0xFF);
}

//	------------------------------	GetNumTracks

short
CDGetNumTracks()
{
ParamBlockRec	params;
OSErr		theErr;
int			loop;
char			lastTrack;
	
	params.cntrlParam.ioCompletion = nil;
	params.cntrlParam.ioVRefNum = 1;
	params.cntrlParam.ioCRefNum = gCDDriverRef;
	params.cntrlParam.csCode = csReadTOC;
	params.cntrlParam.csParam[0] = 1;					//	Type 1 TOC lookup marker

	//	CD-ROM driver ref says we should zero the rest of the fields out.  Probably don't need to
	//	but why takes chances?
	for (loop = 1; loop < 11; loop++)
		params.cntrlParam.csParam[loop] = 0;
	
	theErr = PBControlSync(&params);
	if (theErr)
		return (0);

	lastTrack = params.cntrlParam.csParam[0] & 0xFF;

	//	Return the number of tracks in decimal instead of BCD form
	return (BCDToNum(lastTrack));
}
